#include "system.h"
#include "intel_fpga_i2c.h"
#include "io.h"
#include "stdio.h"

static unsigned char TI_REG_BANK [32];

void intel_fpga_i2c_init(long i2c_base, int sys_clk) {
  // Disable core (also puts it in 100kHz mode) 
  IOWR(i2c_base, I2C_CTRL, 0);

  // Disabling all interrupts as we do not use them
  IOWR(i2c_base, I2C_ISER, 0);
  
  // Setup scl count in terms of system clock cycles (split in half for high and low portions)
  // Assuming 100kHz on i2c
  int scl_count = (sys_clk + 99000)/100000;
  IOWR(i2c_base, I2C_SCL_LOW, scl_count/2);
  IOWR(i2c_base, I2C_SCL_HIGH, (scl_count+1)/2);
  
  //set SDA hold to roughly half scl low time
  IOWR(i2c_base, I2C_SDA_HOLD, scl_count/4);
  
  // Enable core
  IOWR(i2c_base, I2C_CTRL, 1);
}

void intel_fpga_i2c_write_simple(long i2c_base, unsigned char address, unsigned char data) {
  intel_fpga_i2c_wait_busy(i2c_base);
  
  // write address
  unsigned int tx_word = address << 1;
  tx_word = tx_word | I2C_WRITE_FLAG | I2C_START_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);

  // write data
  tx_word = data | I2C_STOP_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);

  intel_fpga_i2c_wait_busy(i2c_base);
}


unsigned char intel_fpga_i2c_read_simple(long i2c_base, unsigned char address) {
  intel_fpga_i2c_wait_busy(i2c_base);

  // write address for reading
  unsigned int tx_word = address << 1;
  tx_word = tx_word | I2C_READ_FLAG | I2C_START_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);
  
  // read data
  tx_word = I2C_STOP_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);
  
  intel_fpga_i2c_wait_busy(i2c_base);
  while(intel_fpga_i2c_rx_data_fill(i2c_base) == 0) {}
  
  return IORD(i2c_base, I2C_RX_DATA);
}


void intel_fpga_i2c_write_extended(long i2c_base, unsigned char address, unsigned char reg, unsigned char data) {
  intel_fpga_i2c_wait_busy(i2c_base);
  // write address
  unsigned int tx_word = address << 1;
  tx_word = tx_word | I2C_WRITE_FLAG | I2C_START_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);

  // write register address
  tx_word = reg;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);
  

  // write data
  tx_word = data | I2C_STOP_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);
  
  intel_fpga_i2c_wait_busy(i2c_base);

}

unsigned char intel_fpga_i2c_read_extended(long i2c_base, unsigned char address, unsigned char reg) {
  intel_fpga_i2c_wait_busy(i2c_base);
  
  // write address
  unsigned int tx_word = address << 1;
  tx_word = tx_word | I2C_WRITE_FLAG | I2C_START_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);

  // write register address
  tx_word = reg;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);

  // write address for reading
  tx_word = address << 1;
  tx_word = tx_word | I2C_READ_FLAG | I2C_START_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);
  
  // read data
  tx_word = I2C_STOP_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);
  
  intel_fpga_i2c_wait_busy(i2c_base);
  while(intel_fpga_i2c_rx_data_fill(i2c_base) == 0) {}
  
  return IORD(i2c_base, I2C_RX_DATA);
}

void intel_fpga_i2c_burst_read_extended(long i2c_base, unsigned char address, unsigned char reg, unsigned char burst_size, unsigned char *data){
  intel_fpga_i2c_wait_busy(i2c_base);
  
  // write address
  unsigned int tx_word = address << 1;
  tx_word = tx_word | I2C_WRITE_FLAG | I2C_START_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);

  // write register address
  tx_word = reg;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);

  // write address for reading
  tx_word = address << 1;
  tx_word = tx_word | I2C_READ_FLAG | I2C_START_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);
  
  for (int i=0; i<burst_size; i++) {
     // read data
     if (i==burst_size-1) {
         tx_word = I2C_STOP_FLAG;
     } else {
         tx_word = 0;
     }
     IOWR(i2c_base, I2C_TFR_CMD, tx_word);
     
     intel_fpga_i2c_wait_busy(i2c_base);
     while(intel_fpga_i2c_rx_data_fill(i2c_base) == 0) {}
     
     *(data+i) = IORD(i2c_base, I2C_RX_DATA);
  }
}

void intel_fpga_i2c_burst_write_extended(long i2c_base, unsigned char address, unsigned char reg, unsigned char burst_size, unsigned char *data){
  intel_fpga_i2c_wait_busy(i2c_base);
  // write address
  unsigned int tx_word = address << 1;
  tx_word = tx_word | I2C_WRITE_FLAG | I2C_START_FLAG;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);

  // write register address
  tx_word = reg;
  IOWR(i2c_base, I2C_TFR_CMD, tx_word);
  

  // write data
  
  for (int i=0; i<burst_size; i++) {
     // read data
     if (i==burst_size-1) {
         tx_word = I2C_STOP_FLAG;
     } else {
         tx_word = *(data+i) | I2C_STOP_FLAG;
     }
     IOWR(i2c_base, I2C_TFR_CMD, tx_word);
     
     intel_fpga_i2c_wait_busy(i2c_base);
     
  }
}

void intel_fpga_i2c_wait_busy(long i2c_base) {
  while(intel_fpga_i2c_tfr_cmd_fill(i2c_base) > 0) { 
  //printf ("intel_fpga_i2c_tfr_cmd_fill=%d\n", intel_fpga_i2c_tfr_cmd_fill(i2c_base)); 
  }
}

unsigned int intel_fpga_i2c_tfr_cmd_fill(long i2c_base) {
   return IORD(i2c_base, I2C_TFR_CMD_FIFO_LEVEL);
}
unsigned int intel_fpga_i2c_rx_data_fill(long i2c_base) {
   return IORD(i2c_base, I2C_RX_DATA_FIFO_LEVEL);
}

void ti_i2c_write_regs(unsigned int base, unsigned int addr, unsigned int reg, unsigned int val)
{
		intel_fpga_i2c_write_extended(base, addr >> 1, reg, val);
		for(int i=0;i<0xffff;i++);
    intel_fpga_i2c_burst_read_extended(base, addr >> 1, 0x00, NBR_OF_TI_REGISTERS, TI_REG_BANK);

}


void ti_i2c_read_regs(unsigned int base, unsigned int addr)
{
	unsigned int i;

	intel_fpga_i2c_burst_read_extended(base, addr >> 1, 0x00, NBR_OF_TI_REGISTERS, TI_REG_BANK);

	for(i=0;i<NBR_OF_TI_REGISTERS;i++){
	  printf("reg[%02X] = %02X\n", i, TI_REG_BANK[i]);
	}

}




